function SystemMenuMessage(id) {
	this.messagetype = "systemmenu";

	this.id = id;
};

function WebConnection(path) {
	this.dialogElement = "";
	this.contentElement = "";
	this.systemMenuElement = "";
	this.showConnectionCloseError = true;
	this.host = window.location.hostname;
	this.port = window.location.port;
	this.path = path;
	this._socket = null;
	this._closeDialog = "";
	this._image = undefined;
	
	this.logout = function () {
		this.showConnectionCloseError = false;
		document.cookie = "hash=;-1;" + "; path=/";
		document.cookie = "user=;-1;" + "; path=/";
		document.cookie = "sid=;-1;" + "; path=/";
		window.location.replace("/");
	};
	
	this.onNotification = function (type, text) {
		noty({ text: text, layout: 'topRight', type: (type === "1" || type === "0") ? 'success' : 'error', timeout: 10000 });
	};
	
	this.onError = function (message) {
		noty({ text: message, layout: 'topRight', type: 'error', timeout: 10000 });
	};
	
	this.onImage = function (receiver, type, image) {};
	
	this._onOpen = function () {};
	this._onClose = function () { return false; };
	this._onMessage = function (message) { return false; };
	this._onBlob = function (data) { return false; };
	
	this.onCommand = function (message) {};
	this._onCommand = function (message) {
		switch (message.command) {
		case "onconnectionclose":
			this._closeDialog = message.param;
			break;
		case "logout":
			this.logout();
			break;
		case "systemmenu":
			$(this.systemMenuElement).html(message.param);
			break;
		case "opendialog":
			$(this.dialogElement).append(message.param);
			break;
		case "refresh":
			location.reload(true);
			break;
		case "deletecookie":
			document.cookie = message.cookiename + '=;-1;';
			break;
		case "setcookie":
			document.cookie = message.cookiename + "=" + message.cookievalue + "; path=/";
			break;
		case "redirect":
			this.showConnectionCloseError = false;
			window.location.replace(message.param);
			break;
		case "openlink":
			window.open(message.param);
			break;
		case "saveas":
			var blob = new Blob([message.param], {type: "text/plain;charset=utf-8"});
			saveAs(blob, message.param2);
			break;
		case "settitle":
			$("title").text(message.param);
			break;
		case "openpage":
			var oldTab = ($(".ui-tabs").length === 1) ? $(".ui-tabs").tabs("option", "active") : -1;
            var oldWindow = $(".window").attr("data-id");
			var scrollPos = $(window).scrollTop();
			$(this.contentElement).html(message.param);
			if ($(".ui-tabs").length === 1 && oldTab !== -1 && oldWindow === $(".window").attr("data-id")) {
				$(".ui-tabs").tabs("option", "active", oldTab);
			}
			$(window).scrollTop(scrollPos);
			break;
		case "openframe":
			$(this.contentElement).empty();
			$(this.contentElement).append("<iframe src='" + message.param + "?rand=" + Math.random() + "' width='100%' height='1000px' seamless='seamless' frameborder='0'></iframe>");
			break;
		case "clear":
			$(this.contentElement).html("");
			break;
		case "back":
			window.history.back();
			break;
		default: return false;
		};
		return true;
	};
		
	this._send = function (message) {
		this._socket.send(JSON.stringify(message, null, 2));
	};

	this.onConnectionClosed = function () {};
	this.isConnected = function () { return this._socket && this._socket.readyState === 1; };

	this.start = function () {
		var that = this;
		var imageTarget = "";
		
		// Jos portti on vakioportti (80, 443), joissain selaimissa (IE) window.location.port on tyhjä
		this._socket = new WebSocket((location.protocol === "https:" ? "wss" : "ws") + "://" + this.host + (this.port.length === 0 ? "" : ":" + this.port) + this.path);
		this._socket.onopen = function () { that._onOpen(); };
		this._socket.onclose = function () {
			if (that._onClose() || !that.showConnectionCloseError) return;
			if (that._closeDialog.length !== 0) {
				that.onConnectionClosed();
				$(that.dialogElement).html(that._closeDialog);
			}
		};
		this._socket.onmessage = function (event) {
			if (typeof event.data !== "string") {
				if (that._onBlob(event.data)) return;
				if (imageTarget.length === 0) return;
				that.onImage(imageTarget, "binary", event.data);
				imageTarget = "";
			}

			var result;
			try {
				result = jQuery.parseJSON(event.data);
			} catch(e) {
				that.onError("Failed to parse reply: " + e);
				return;
			}

			// Annetaan perivälle luokalle mahdollisuus käsitellä raakaviesti
			if (result === null || that._onMessage(result)) return;

			var type = result.type;
			var message = result.message;

			if (type === "command") {
				if (!that._onCommand(message)) that.onCommand(message);
			} else if (type === "image") {
				if (message.encoding === "base64") that.onImage(message.target, "base64", message.data);
				else imageTarget = message.target;
			} else if (type === "notification") {
				that.onNotification(message.type, message.text);
			} else if (type === "window") {
				processWindowMessage(message);
			}
		};
	}

	this.systemMenu = function (id) { this._send(new SystemMenuMessage(id)); };
};
